@@ -1,10 +1,16 @@
 import io
 import sys
 from types import ModuleType
+from typing import Sequence, Type
 
 import pytest
 
 from rich import inspect
+from rich._inspect import (
+    get_object_types_mro,
+    get_object_types_mro_as_strings,
+    is_object_one_of_types,
+)
 from rich.console import Console
 
 skip_py36 = pytest.mark.skipif(
@@ -32,6 +38,11 @@ skip_py310 = pytest.mark.skipif(
     reason="rendered differently on py3.10",
 )
 
+skip_py311 = pytest.mark.skipif(
+    sys.version_info.minor == 11 and sys.version_info.major == 3,
+    reason="rendered differently on py3.11",
+)
+
 skip_pypy3 = pytest.mark.skipif(
     hasattr(sys, "pypy_version_info"),
     reason="rendered differently on pypy3",
@@ -74,6 +85,10 @@ class Foo:
         return ["__init__", "broken", "method"]
 
 
+class FooSubclass(Foo):
+    pass
+
+
 @skip_py36
 def test_render():
     console = Console(width=100, file=io.StringIO(), legacy_windows=False)
@@ -83,23 +98,24 @@ def test_render():
     result = console.file.getvalue()
     print(repr(result))
     expected = "╭────────────── <class 'tests.test_inspect.Foo'> ──────────────╮\n│ Foo test                                                     │\n│                                                              │\n│   broken = InspectError()                                    │\n│ __init__ = def __init__(foo: int) -> None: constructor docs. │\n│   method = def method(a, b) -> str: Multi line               │\n╰──────────────────────────────────────────────────────────────╯\n"
-    assert expected == result
+    assert result == expected
 
 
 @skip_pypy3
 def test_inspect_text():
+    num_attributes = 34 if sys.version_info >= (3, 11) else 33
     expected = (
         "╭──────────────── <class 'str'> ─────────────────╮\n"
         "│ str(object='') -> str                          │\n"
         "│ str(bytes_or_buffer[, encoding[, errors]]) ->  │\n"
         "│ str                                            │\n"
         "│                                                │\n"
-        "│ 33 attribute(s) not shown. Run                 │\n"
+        f"│ {num_attributes} attribute(s) not shown. Run                 │\n"
         "│ inspect(inspect) for options.                  │\n"
         "╰────────────────────────────────────────────────╯\n"
     )
     print(repr(expected))
-    assert expected == render("Hello")
+    assert render("Hello") == expected
 
 
 @skip_py36
@@ -126,8 +142,10 @@ def test_inspect_empty_dict():
     assert render({}).startswith(expected)
 
 
+@skip_py311
 @skip_pypy3
-def test_inspect_builtin_function():
+def test_inspect_builtin_function_except_python311():
+    # Pre-3.11 Python versions - print builtin has no signature available
     expected = (
         "╭────────── <built-in function print> ───────────╮\n"
         "│ def print(...)                                 │\n"
@@ -139,7 +157,41 @@ def test_inspect_builtin_function():
         "│ inspect(inspect) for options.                  │\n"
         "╰────────────────────────────────────────────────╯\n"
     )
-    assert expected == render(print)
+    assert render(print) == expected
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 11), reason="print builtin signature only available on 3.11+"
+)
+@skip_pypy3
+def test_inspect_builtin_function_only_python311():
+    # On 3.11, the print builtin *does* have a signature, unlike in prior versions
+    expected = (
+        "╭────────── <built-in function print> ───────────╮\n"
+        "│ def print(*args, sep=' ', end='\\n', file=None, │\n"
+        "│ flush=False):                                  │\n"
+        "│                                                │\n"
+        "│ Prints the values to a stream, or to           │\n"
+        "│ sys.stdout by default.                         │\n"
+        "│                                                │\n"
+        "│ 30 attribute(s) not shown. Run                 │\n"
+        "│ inspect(inspect) for options.                  │\n"
+        "╰────────────────────────────────────────────────╯\n"
+    )
+    assert render(print) == expected
+
+
+@skip_pypy3
+def test_inspect_coroutine():
+    async def coroutine():
+        pass
+
+    expected = (
+        "╭─ <function test_inspect_coroutine.<locals>.cor─╮\n"
+        "│ async def                                      │\n"
+        "│ test_inspect_coroutine.<locals>.coroutine():   │\n"
+    )
+    assert render(coroutine).startswith(expected)
 
 
 @skip_py36
@@ -163,13 +215,14 @@ def test_inspect_integer_with_value():
     expected = "╭────── <class 'int'> ───────╮\n│ int([x]) -> integer        │\n│ int(x, base=10) -> integer │\n│                            │\n│ ╭────────────────────────╮ │\n│ │ 1                      │ │\n│ ╰────────────────────────╯ │\n│                            │\n│ denominator = 1            │\n│        imag = 0            │\n│   numerator = 1            │\n│        real = 1            │\n╰────────────────────────────╯\n"
     value = render(1, value=True)
     print(repr(value))
-    assert expected == value
+    assert value == expected
 
 
 @skip_py36
 @skip_py37
 @skip_py310
-def test_inspect_integer_with_methods():
+@skip_py311
+def test_inspect_integer_with_methods_python38_and_python39():
     expected = (
         "╭──────────────── <class 'int'> ─────────────────╮\n"
         "│ int([x]) -> integer                            │\n"
@@ -199,14 +252,15 @@ def test_inspect_integer_with_methods():
         "│                    an integer.                 │\n"
         "╰────────────────────────────────────────────────╯\n"
     )
-    assert expected == render(1, methods=True)
+    assert render(1, methods=True) == expected
 
 
 @skip_py36
 @skip_py37
 @skip_py38
 @skip_py39
-def test_inspect_integer_with_methods():
+@skip_py311
+def test_inspect_integer_with_methods_python310only():
     expected = (
         "╭──────────────── <class 'int'> ─────────────────╮\n"
         "│ int([x]) -> integer                            │\n"
@@ -240,7 +294,51 @@ def test_inspect_integer_with_methods():
         "│                    an integer.                 │\n"
         "╰────────────────────────────────────────────────╯\n"
     )
-    assert expected == render(1, methods=True)
+    assert render(1, methods=True) == expected
+
+
+@skip_py36
+@skip_py37
+@skip_py38
+@skip_py39
+@skip_py310
+def test_inspect_integer_with_methods_python311_and_above():
+    # to_bytes and from_bytes methods on int had minor signature change -
+    # they now, as of 3.11, have default values for all of their parameters
+    expected = (
+        "╭──────────────── <class 'int'> ─────────────────╮\n"
+        "│ int([x]) -> integer                            │\n"
+        "│ int(x, base=10) -> integer                     │\n"
+        "│                                                │\n"
+        "│      denominator = 1                           │\n"
+        "│             imag = 0                           │\n"
+        "│        numerator = 1                           │\n"
+        "│             real = 1                           │\n"
+        "│ as_integer_ratio = def as_integer_ratio():     │\n"
+        "│                    Return integer ratio.       │\n"
+        "│        bit_count = def bit_count(): Number of  │\n"
+        "│                    ones in the binary          │\n"
+        "│                    representation of the       │\n"
+        "│                    absolute value of self.     │\n"
+        "│       bit_length = def bit_length(): Number of │\n"
+        "│                    bits necessary to represent │\n"
+        "│                    self in binary.             │\n"
+        "│        conjugate = def conjugate(...) Returns  │\n"
+        "│                    self, the complex conjugate │\n"
+        "│                    of any int.                 │\n"
+        "│       from_bytes = def from_bytes(bytes,       │\n"
+        "│                    byteorder='big', *,         │\n"
+        "│                    signed=False): Return the   │\n"
+        "│                    integer represented by the  │\n"
+        "│                    given array of bytes.       │\n"
+        "│         to_bytes = def to_bytes(length=1,      │\n"
+        "│                    byteorder='big', *,         │\n"
+        "│                    signed=False): Return an    │\n"
+        "│                    array of bytes representing │\n"
+        "│                    an integer.                 │\n"
+        "╰────────────────────────────────────────────────╯\n"
+    )
+    assert render(1, methods=True) == expected
 
 
 @skip_py36
@@ -299,3 +397,102 @@ def test_inspect_module_with_class():
         "╰──────────────────────────────────────────╯\n"
     )
     assert render(module, methods=True) == expected
+
+
+@pytest.mark.parametrize(
+    "special_character,expected_replacement",
+    (
+        ("\a", "\\a"),
+        ("\b", "\\b"),
+        ("\f", "\\f"),
+        ("\r", "\\r"),
+        ("\v", "\\v"),
+    ),
+)
+def test_can_handle_special_characters_in_docstrings(
+    special_character: str, expected_replacement: str
+) -> None:
+    class Something:
+        class Thing:
+            pass
+
+    Something.Thing.__doc__ = f"""
+    Multiline docstring
+    with {special_character} should be handled
+    """
+
+    expected = """\
+╭─ <class 'tests.test_inspect.test_can_handle_sp─╮
+│ class                                          │
+│ test_can_handle_special_characters_in_docstrin │
+│ gs.<locals>.Something():                       │
+│                                                │
+│ Thing = class Thing():                         │
+│         Multiline docstring                    │
+│         with %s should be handled              │
+╰────────────────────────────────────────────────╯
+""" % (
+        expected_replacement
+    )
+    assert render(Something, methods=True) == expected
+
+
+@pytest.mark.parametrize(
+    "obj,expected_result",
+    (
+        [object, (object,)],
+        [object(), (object,)],
+        ["hi", (str, object)],
+        [str, (str, object)],
+        [Foo(1), (Foo, object)],
+        [Foo, (Foo, object)],
+        [FooSubclass(1), (FooSubclass, Foo, object)],
+        [FooSubclass, (FooSubclass, Foo, object)],
+    ),
+)
+def test_object_types_mro(obj: object, expected_result: Sequence[Type]):
+    assert get_object_types_mro(obj) == expected_result
+
+
+@pytest.mark.parametrize(
+    "obj,expected_result",
+    (
+        # fmt: off
+        ["hi", ["builtins.str", "builtins.object"]],
+        [str, ["builtins.str", "builtins.object"]],
+        [Foo(1), [f"{__name__}.Foo", "builtins.object"]],
+        [Foo, [f"{__name__}.Foo", "builtins.object"]],
+        [FooSubclass(1),
+         [f"{__name__}.FooSubclass", f"{__name__}.Foo", "builtins.object"]],
+        [FooSubclass,
+         [f"{__name__}.FooSubclass", f"{__name__}.Foo", "builtins.object"]],
+        # fmt: on
+    ),
+)
+def test_object_types_mro_as_strings(obj: object, expected_result: Sequence[str]):
+    assert get_object_types_mro_as_strings(obj) == expected_result
+
+
+@pytest.mark.parametrize(
+    "obj,types,expected_result",
+    (
+        # fmt: off
+        ["hi", ["builtins.str"], True],
+        [str, ["builtins.str"], True],
+        ["hi", ["builtins.str", "foo"], True],
+        [str, ["builtins.str", "foo"], True],
+        [Foo(1), [f"{__name__}.Foo"], True],
+        [Foo, [f"{__name__}.Foo"], True],
+        [Foo(1), ["builtins.str", f"{__name__}.Foo"], True],
+        [Foo, ["builtins.int", f"{__name__}.Foo"], True],
+        [Foo(1), [f"{__name__}.FooSubclass"], False],
+        [Foo, [f"{__name__}.FooSubclass"], False],
+        [Foo(1), [f"{__name__}.FooSubclass", f"{__name__}.Foo"], True],
+        [Foo, [f"{__name__}.Foo", f"{__name__}.FooSubclass"], True],
+        # fmt: on
+    ),
+)
+def test_object_is_one_of_types(
+    obj: object, types: Sequence[str], expected_result: bool
+):
+    assert is_object_one_of_types(obj, types) is expected_result
